from distutils.core import setup
setup(
  name = 'MorseStation',
  packages = ['MorseStation'],
  version = '1.0',
  description = 'Tool for converting A-Z 0-9 messages into Morse code Wave files',
  author = 'Karl Zander',
  author_email = 'pvial@kryptomagik.com',
  url = 'https://github.com/pvial00/MorseStation',
  keywords = ['cryptography', 'encryption', 'security'],
  classifiers = [],
  install_requires=[
      'KlassiKrypto',
      ],
)



from bleeps import Bleeps
from KlassiKrypto import Morse
import datetime

class MorseStation:
    # preamble True enable the internal preable
    def __init__(self, frequency=1000, wpm=60, timeunit=1.2, volume=1, preamble=False, spacing=5):
        self.timeunit = timeunit
        self.wpm = wpm
        self.frequency = frequency
        self.volume = volume
        self.preamble = preamble
        self.spacing = spacing

    def farnsworth2_1(self):
        unit = float(self.timeunit / self.wpm)
        dot_wait = unit * 1000
        dash_wait = (unit * 3) * 1000
        element_wait = (unit * 1000)
        letter_wait = (unit * 3) * 1000
        word_wait= (unit * 12) * 1000
        return dot_wait, dash_wait, element_wait, letter_wait, word_wait

    # Create a Morse code WAV file 
    # Accepts A-Z 0-9 as input
    # preamble and eot (end of transmission are used if one wants and always overrides the internal preamble
    def transmit(self, data, filename, preamble="", eot="", msgnumber=0, station=""):
        if self.preamble == True and preamble == "" and eot == "":
           if msgnumber != 0:
               msgn = str(msgnumber)
           else:
               msgn = ""
           preamble = station + "MSG" + msgn + "CK" + str(len(data)) + "CK" + datetime.datetime.now().strftime('%Y%M%d%H%M') + "BT"
           eot = "000"

        code = Morse().encode(preamble+data+eot)
        bleeps = Bleeps()
        letters = code.split()
        dot_wait, dash_wait, element_wait, letter_wait, word_wait = self.farnsworth2_1()
        count = 0
        for c, letter in enumerate(letters):
            count += 1
            for element in letter:
                if element == ".":
                    bleeps.append_sinewave(freq=self.frequency,duration_milliseconds=dot_wait,volume=self.volume)
                elif element == "-":
                    bleeps.append_sinewave(freq=self.frequency,duration_milliseconds=dash_wait,volume=self.volume)
                bleeps.append_silence(duration_milliseconds=element_wait)
            if (count == self.spacing) and self.spacing != 0 and self.spacing != 1:
                bleeps.append_silence(duration_milliseconds=word_wait)
                count = 0
            else:
                bleeps.append_silence(duration_milliseconds=letter_wait)
        bleeps.save_wave(filename)



#!/usr/bin/python 
# based on : www.daniweb.com/code/snippet263775.html
# and based on : https://stackoverflow.com/questions/33879523/python-how-can-i-generate-a-wav-file-with-beeps
import math
import wave
import struct

# Audio will contain a long list of samples (i.e. floating point numbers describing the
# waveform).  If you were working with a very long sound you'd want to stream this to
# disk instead of buffering it all in memory list this.  But most sounds will fit in 
# memory.
class Bleeps:
    def __init__(self, sample_rate=44100.0):
        self.audio = []
        self.sample_rate = sample_rate

    def append_silence(self, duration_milliseconds=500):
        """
        Adding silence is easy - we add zeros to the end of our array
        """
        num_samples = duration_milliseconds * (self.sample_rate / 1000.0)

        entry = []
        for x in range(int(num_samples)): 
            entry.append(0.0)
        self.audio.append(entry)


    def append_sinewave(self,
            freq=440.0, 
            duration_milliseconds=500, 
            volume=1.0):
        """
        The sine wave generated here is the standard beep.  If you want something
        more aggresive you could try a square or saw tooth waveform.   Though there
        are some rather complicated issues with making high quality square and
        sawtooth waves... which we won't address here :) 
        """ 

        num_samples = duration_milliseconds * (self.sample_rate / 1000.0)

        entry = []
        for x in range(int(num_samples)):
            entry.append((volume * math.sin(2 * math.pi * freq * ( x / self.sample_rate ))))
        self.audio.append(entry)
            #self.audio.append(entry.append((volume * math.sin(2 * math.pi * freq * ( x / self.sample_rate )))))

    def save_wave(self, file_name):
        # Open up a wav file
        wav_file=wave.open(file_name,"w")

        # wav params
        nchannels = 1

        sampwidth = 2

        # 44100 is the industry standard sample rate - CD quality.  If you need to
        # save on file size you can adjust it downwards. The stanard for low quality
        # is 8000 or 8kHz.
        nframes = len(self.audio)
        comptype = "NONE"
        compname = "not compressed"
        wav_file.setparams((nchannels, sampwidth, self.sample_rate, nframes, comptype, compname))

        # WAV files here are using short, 16 bit, signed integers for the 
        # sample size.  So we multiply the floating point data we have by 32767, the
        # maximum value for a short integer.  NOTE: It is theortically possible to
        # use the floating point -1.0 to 1.0 data directly in a WAV file but not
        # obvious how to do that using the wave module in python.
        for sample in self.audio:
            for entry in sample:
                wav_file.writeframes(struct.pack('h', int( entry * 32767.0 )))

        wav_file.close()
